@@ -16,6 +16,7 @@ Wrapper for pywsman.Client
 """
 
 import logging
+import subprocess
 import time
 
 from dracclient import constants
@@ -32,7 +33,7 @@ from dracclient.resources import uris
 from dracclient import utils
 from dracclient import wsman
 
-IDRAC_IS_READY = "LC061"
+IDRAC_IS_READY = "0"
 
 LOG = logging.getLogger(__name__)
 
@@ -174,11 +175,6 @@ class DRACClient(object):
                          each key being the name of attribute and the value
                          being the proposed value.
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with a boolean value indicating
                    whether a config job must be created for the values to be
                    applied.
@@ -243,6 +239,103 @@ class DRACClient(object):
         """
         return self._idrac_cfg.set_idrac_settings(settings, idrac_fqdd)
 
+    def reset_idrac(self, force=False, wait=False,
+                    ready_wait_time=30):
+        """Resets the iDRAC and optionally block until reset is complete.
+
+        :param force: does a force reset when True and a graceful reset when
+               False
+        :param wait: returns immediately after reset if False, or waits
+                for the iDRAC to return to operational state if True
+        :param ready_wait_time: the amount of time in seconds to wait after
+                the reset before starting to check on the iDRAC's status
+        :returns: True on success, raises exception on failure
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on failure to reset iDRAC
+        """
+        return_value = self._idrac_cfg.reset_idrac(force)
+        if not wait and return_value:
+            return return_value
+
+        if not return_value:
+            raise exceptions.DRACOperationFailed(
+                drac_messages="Failed to reset iDRAC")
+
+        LOG.debug("iDRAC was reset, waiting for return to operational state")
+
+        state_reached = self._wait_for_host_state(
+            self.client.host,
+            alive=False,
+            ping_count=2,
+            retries=24)
+
+        if not state_reached:
+            raise exceptions.DRACOperationFailed(
+                drac_messages="Timed out waiting for the %s iDRAC to become "
+                "not pingable" % self.client.host)
+
+        LOG.info("The iDRAC has become not pingable")
+
+        state_reached = self._wait_for_host_state(
+            self.client.host,
+            alive=True,
+            ping_count=3,
+            retries=24)
+
+        if not state_reached:
+            raise exceptions.DRACOperationFailed(
+                drac_messages="Timed out waiting for the %s iDRAC to become "
+                "pingable" % self.client.host)
+
+        LOG.info("The iDRAC has become pingable")
+        LOG.info("Waiting for the iDRAC to become ready")
+        time.sleep(ready_wait_time)
+
+        self.client.wait_until_idrac_is_ready()
+
+    def _ping_host(self, host):
+        response = subprocess.call(
+            "ping -c 1 {} 2>&1 1>/dev/null".format(host), shell=True)
+        return (response == 0)
+
+    def _wait_for_host_state(self,
+                             host,
+                             alive=True,
+                             ping_count=3,
+                             retries=24):
+        if alive:
+            ping_type = "pingable"
+
+        else:
+            ping_type = "not pingable"
+
+        LOG.info("Waiting for the iDRAC to become %s", ping_type)
+
+        response_count = 0
+        state_reached = False
+
+        while retries > 0 and not state_reached:
+            response = self._ping_host(host)
+            retries -= 1
+            if response == alive:
+                response_count += 1
+                LOG.debug("The iDRAC is %s, count=%s",
+                          ping_type,
+                          response_count)
+                if response_count == ping_count:
+                    LOG.debug("Reached specified ping count")
+                    state_reached = True
+            else:
+                response_count = 0
+                if alive:
+                    LOG.debug("The iDRAC is still not pingable")
+                else:
+                    LOG.debug("The iDRAC is still pingable")
+            time.sleep(10)
+
+        return state_reached
+
     def commit_pending_idrac_changes(
             self,
             idrac_fqdd=IDRAC_FQDD,
@@ -290,9 +383,12 @@ class DRACClient(object):
             cim_name='DCIM:iDRACCardService',
             target=idrac_fqdd)
 
-    def list_lifecycle_settings(self):
+    def list_lifecycle_settings(self, by_name=False):
         """List the Lifecycle Controller configuration settings
 
+        :param by_name: Controls whether returned dictionary uses Lifecycle
+                        attribute name as key. If set to False, instance_id
+                        will be used.
         :returns: a dictionary with the Lifecycle Controller settings using its
                   InstanceID as the key. The attributes are either
                   LCEnumerableAttribute or LCStringAttribute objects.
@@ -301,7 +397,49 @@ class DRACClient(object):
         :raises: DRACOperationFailed on error reported back by the DRAC
                  interface
         """
-        return self._lifecycle_cfg.list_lifecycle_settings()
+        return self._lifecycle_cfg.list_lifecycle_settings(by_name)
+
+    def is_lifecycle_in_recovery(self):
+        """Checks if Lifecycle Controller in recovery mode or not
+
+        This method checks the LCStatus value to determine if lifecycle
+        controller is in recovery mode by invoking GetRemoteServicesAPIStatus
+        from iDRAC.
+
+        :returns: a boolean indicating if lifecycle controller is in recovery
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        """
+
+        return self._lifecycle_cfg.is_lifecycle_in_recovery()
+
+    def set_lifecycle_settings(self, settings):
+        """Sets lifecycle controller configuration
+
+        It sets the pending_value parameter for each of the attributes
+        passed in. For the values to be applied, a config job must
+        be created.
+
+        :param settings: a dictionary containing the proposed values, with
+                         each key being the name of attribute and the value
+                         being the proposed value.
+        :returns: a dictionary containing:
+                 - The is_commit_required key with a boolean value indicating
+                   whether a config job must be created for the values to be
+                   applied.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted for the
+                   values to be applied. Possible values are true and false.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        :raises: InvalidParameterValue on invalid Lifecycle attribute
+        """
+        return self._lifecycle_cfg.set_lifecycle_settings(settings)
 
     def list_system_settings(self):
         """List the System configuration settings
@@ -370,7 +508,10 @@ class DRACClient(object):
                           cim_system_creation_class_name='DCIM_ComputerSystem',
                           cim_system_name='DCIM:ComputerSystem',
                           reboot=False,
-                          start_time='TIME_NOW'):
+                          start_time='TIME_NOW',
+                          realtime=False,
+                          wait_for_idrac=True,
+                          method_name='CreateTargetedConfigJob'):
         """Creates a configuration job.
 
         In CIM (Common Information Model), weak association is used to name an
@@ -394,6 +535,12 @@ class DRACClient(object):
                            means execute immediately or None which means
                            the job will not execute until
                            schedule_job_execution is called
+        :param realtime: Indicates if reatime mode should be used.
+               Valid values are True and False.
+        :param wait_for_idrac: indicates whether or not to wait for the
+                               iDRAC to be ready to accept commands before
+                               issuing the command.
+        :param method_name: method of CIM object to invoke
         :returns: id of the created job
         :raises: WSManRequestFailure on request failures
         :raises: WSManInvalidResponse when receiving invalid response
@@ -410,7 +557,10 @@ class DRACClient(object):
             cim_system_creation_class_name=cim_system_creation_class_name,
             cim_system_name=cim_system_name,
             reboot=reboot,
-            start_time=start_time)
+            start_time=start_time,
+            realtime=realtime,
+            wait_for_idrac=wait_for_idrac,
+            method_name=method_name)
 
     def create_nic_config_job(
             self,
@@ -549,6 +699,37 @@ class DRACClient(object):
             cim_creation_class_name='DCIM_BIOSService',
             cim_name='DCIM:BIOSService', target=self.BIOS_DEVICE_FQDD)
 
+    def commit_pending_lifecycle_changes(
+            self,
+            reboot=False,
+            start_time='TIME_NOW'):
+        """Applies all pending changes on Lifecycle by creating a config job
+
+        :param reboot: indicates whether a RebootJob should also be
+                       created or not
+        :param start_time: start time for job execution in format
+                           yyyymmddhhmmss, the string 'TIME_NOW' which
+                           means execute immediately or None which means
+                           the job will not execute until
+                           schedule_job_execution is called
+        :returns: id of the created job
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface, including start_time being in the past or
+                 badly formatted start_time
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        """
+        return self._job_mgmt.create_config_job(
+            resource_uri=uris.DCIM_LCService,
+            cim_creation_class_name='DCIM_LCService',
+            cim_name='DCIM:LCService',
+            target='',
+            reboot=reboot,
+            start_time=start_time,
+            wait_for_idrac=False,
+            method_name='CreateConfigJob')
+
     def get_lifecycle_controller_version(self):
         """Returns the Lifecycle controller version
 
@@ -607,11 +788,6 @@ class DRACClient(object):
                become part of the RAID.  The same flag is applied to all
                listed disks
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete disk conversion.
@@ -619,8 +795,8 @@ class DRACClient(object):
                    value indicating whether the server must be rebooted to
                    complete disk conversion.
         """
-        return self._raid_mgmt.convert_physical_disks(
-            physical_disks, raid_enable)
+        return self._raid_mgmt.convert_physical_disks(physical_disks,
+                                                      raid_enable)
 
     def create_virtual_disk(self, raid_controller, physical_disks, raid_level,
                             size_mb, disk_name=None, span_length=None,
@@ -637,11 +813,6 @@ class DRACClient(object):
         :param span_length: number of disks per span (optional)
         :param span_depth: number of spans in virtual disk (optional)
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete virtual disk creation.
@@ -667,11 +838,6 @@ class DRACClient(object):
 
         :param virtual_disk: id of the virtual disk
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete virtual disk deletion.
@@ -686,8 +852,52 @@ class DRACClient(object):
         """
         return self._raid_mgmt.delete_virtual_disk(virtual_disk)
 
+    def reset_raid_config(self, raid_controller):
+        """Delete all the virtual disks and unassign all hot spare physical disks
+
+        The job to reset the RAID controller config will be in pending state.
+        For the changes to be applied, a config job must be created.
+
+        :param raid_controller: id of the RAID controller
+        :returns: a dictionary containing:
+                 - The is_commit_required key with the value always set to
+                   True indicating that a config job must be created to
+                   reset configuration.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted to
+                   reset configuration.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        """
+        return self._raid_mgmt.reset_raid_config(raid_controller)
+
+    def clear_foreign_config(self, raid_controller):
+        """Free up foreign drives
+
+        The job to clear foreign config will be in pending state.
+        For the changes to be applied, a config job must be created.
+
+        :param raid_controller: id of the RAID controller
+        :returns: a dictionary containing:
+                 - The is_commit_required key with the value always set to
+                   True indicating that a config job must be created to
+                   clear foreign configuration.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted to
+                   clear foreign configuration.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        """
+        return self._raid_mgmt.clear_foreign_config(raid_controller)
+
     def commit_pending_raid_changes(self, raid_controller, reboot=False,
-                                    start_time='TIME_NOW'):
+                                    start_time='TIME_NOW', realtime=False):
         """Applies all pending changes on a RAID controller
 
          ...by creating a config job.
@@ -700,6 +910,8 @@ class DRACClient(object):
                means execute immediately or None which means
                the job will not execute until
                schedule_job_execution is called
+        :param realtime: Indicates if reatime mode should be used.
+               Valid values are True and False.
         :returns: id of the created job
         :raises: WSManRequestFailure on request failures
         :raises: WSManInvalidResponse when receiving invalid response
@@ -713,7 +925,8 @@ class DRACClient(object):
             cim_name='DCIM:RAIDService',
             target=raid_controller,
             reboot=reboot,
-            start_time=start_time)
+            start_time=start_time,
+            realtime=realtime)
 
     def abandon_pending_raid_changes(self, raid_controller):
         """Deletes all pending changes on a RAID controller
@@ -732,6 +945,14 @@ class DRACClient(object):
             cim_creation_class_name='DCIM_RAIDService',
             cim_name='DCIM:RAIDService', target=raid_controller)
 
+    def is_realtime_supported(self, raid_controller):
+        """Find if controller supports realtime or not
+
+        :param raid_controller: ID of RAID controller
+        :returns: True or False
+        """
+        return self._raid_mgmt.is_realtime_supported(raid_controller)
+
     def list_cpus(self):
         """Returns the list of CPUs
 
@@ -794,9 +1015,13 @@ class DRACClient(object):
         :param settings: dictionary containing the proposed values, with
                          each key being the name of an attribute and the
                          value being the proposed value
-        :returns: dictionary containing a 'commit_required' key with a
-                  boolean value indicating whether a configuration job
-                  must be created for the new settings to be applied
+        :returns: dictionary containing:
+                  - The is_commit_required key with a boolean value
+                    indicating whether a config job must be created for
+                    the values to be applied.
+                  - The is_reboot_required key with a RebootRequired
+                    enumerated value indicating whether the server must
+                    be rebooted for the values to be applied.
         :raises: WSManRequestFailure on request failures
         :raises: WSManInvalidResponse when receiving invalid response
         :raises: DRACOperationFailed on error reported back by the iDRAC
@@ -854,8 +1079,8 @@ class DRACClient(object):
         """Find out if raid controller supports jbod
 
         :param raid_controller_fqdd: The raid controller's fqdd
-                        being being checked to see if it is jbod
-                        capable.
+                                     being checked to see if it is jbod
+                                     capable.
         :raises: DRACRequestFailed if unable to find any disks in the Ready
                  or non-RAID states
         :raises: DRACOperationFailed on error reported back by the DRAC
@@ -864,6 +1089,79 @@ class DRACClient(object):
         """
         return self._raid_mgmt.is_jbod_capable(raid_controller_fqdd)
 
+    def is_raid_controller(self, raid_controller_fqdd, raid_controllers=None):
+        """Determine if the given controller is a RAID controller
+
+        Since a BOSS controller is a type of RAID controller, this method will
+        return True for both BOSS and RAID controllers.
+
+        :param raid_controller_fqdd: The object's fqdd we are testing to see
+                                     if it is a raid controller or not.
+        :param raid_controllers: A list of RAIDControllers used to check for
+                                 the presence of BOSS cards.  If None, the
+                                 iDRAC will be queried for the list of
+                                 controllers.
+        :returns: boolean, True if the device is a RAID controller,
+                  False if not.
+        """
+        return self._raid_mgmt.is_raid_controller(raid_controller_fqdd,
+                                                  raid_controllers)
+
+    def is_boss_controller(self, raid_controller_fqdd, raid_controllers=None):
+        """Find out if a RAID controller a BOSS card or not
+
+        :param raid_controller_fqdd: The object's fqdd we are testing to see
+                                     if it is a BOSS card or not.
+        :param raid_controllers: A list of RAIDController to scan for presence
+                                 of BOSS card, if None the drac will be queried
+                                 for the list of controllers which will then be
+                                 scanned.
+        :returns: boolean, True if the device is a BOSS card, False if not.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        """
+        return self._raid_mgmt.is_boss_controller(raid_controller_fqdd,
+                                                  raid_controllers)
+
+    def change_physical_disk_state(self, mode,
+                                   controllers_to_physical_disk_ids=None):
+        """Convert disks RAID status
+
+        This method intelligently converts the requested physical disks from
+        RAID to JBOD or vice versa.  It does this by only converting the
+        disks that are not already in the correct state.
+
+        :param mode: constants.RaidStatus enumeration that indicates the mode
+                     to change the disks to.
+        :param controllers_to_physical_disk_ids: Dictionary of controllers and
+               corresponding disk ids to convert to the requested mode.
+        :returns: a dictionary containing:
+                  - conversion_results, a dictionary that maps controller ids
+                    to the conversion results for that controller.  The
+                    conversion results are a dict that contains:
+                    - The is_commit_required key with the value always set to
+                      True indicating that a config job must be created to
+                      complete disk conversion.
+                    - The is_reboot_required key with a RebootRequired
+                      enumerated value indicating whether the server must be
+                      rebooted to complete disk conversion.
+                  Also contained in the main dict are the following key/values,
+                  which are deprecated, should not be used, and will be removed
+                  in a future release:
+                  - is_reboot_required, a boolean stating whether a reboot is
+                    required or not.
+                  - commit_required_ids, a list of controller ids that will
+                    need to commit their pending RAID changes via a config job.
+        :raises: DRACOperationFailed on error reported back by the DRAC and the
+                 exception message does not contain NOT_SUPPORTED_MSG constant.
+        :raises: Exception on unknown error.
+        """
+        return (self._raid_mgmt
+                .change_physical_disk_state(mode,
+                                            controllers_to_physical_disk_ids))
+
 
 class WSManClient(wsman.Client):
     """Wrapper for wsman.Client that can wait until iDRAC is ready
@@ -983,8 +1281,8 @@ class WSManClient(wsman.Client):
                             message_elems]
                 raise exceptions.DRACOperationFailed(drac_messages=messages)
 
-            if (expected_return_value is not None and
-                    return_value != expected_return_value):
+            if (expected_return_value is not None
+                    and return_value != expected_return_value):
                 raise exceptions.DRACUnexpectedReturnValue(
                     expected_return_value=expected_return_value,
                     actual_return_value=return_value)
@@ -1017,11 +1315,11 @@ class WSManClient(wsman.Client):
                              expected_return_value=utils.RET_SUCCESS,
                              wait_for_idrac=False)
 
-        message_id = utils.find_xml(result,
-                                    'MessageID',
-                                    uris.DCIM_LCService).text
+        lc_status = utils.find_xml(result,
+                                   'LCStatus',
+                                   uris.DCIM_LCService).text
 
-        return message_id == IDRAC_IS_READY
+        return lc_status == IDRAC_IS_READY
 
     def wait_until_idrac_is_ready(self, retries=None, retry_delay=None):
         """Waits until the iDRAC is in a ready state
