@@ -12,6 +12,7 @@
 #    under the License.
 
 import collections
+import copy
 import logging
 
 from dracclient import constants
@@ -34,6 +35,11 @@ RAID_LEVELS = {
 
 REVERSE_RAID_LEVELS = dict((v, k) for (k, v) in RAID_LEVELS.items())
 
+RAID_CONTROLLER_IS_REALTIME = {
+    '1': True,
+    '0': False
+}
+
 DISK_RAID_STATUS = {
     '0': 'unknown',
     '1': 'ready',
@@ -71,88 +77,29 @@ PHYSICAL_DISK_BUS_PROTOCOL = {
     '8': 'nvme'
 }
 
-PhysicalDiskTuple = collections.namedtuple(
+PhysicalDisk = collections.namedtuple(
     'PhysicalDisk',
     ['id', 'description', 'controller', 'manufacturer', 'model', 'media_type',
      'interface_type', 'size_mb', 'free_size_mb', 'serial_number',
      'firmware_version', 'status', 'raid_status', 'sas_address',
      'device_protocol'])
 
-
-class PhysicalDisk(PhysicalDiskTuple):
-
-    def __new__(cls, **kwargs):
-        if 'state' in kwargs:
-            LOG.warning('PhysicalDisk.state is deprecated. '
-                        'Use PhysicalDisk.status instead.')
-            kwargs['status'] = kwargs['state']
-            del kwargs['state']
-
-        if 'raid_state' in kwargs:
-            LOG.warning('PhysicalDisk.raid_state is deprecated. '
-                        'Use PhysicalDisk.raid_status instead.')
-            kwargs['raid_status'] = kwargs['raid_state']
-            del kwargs['raid_state']
-
-        return super(PhysicalDisk, cls).__new__(cls, **kwargs)
-
-    @property
-    def state(self):
-        LOG.warning('PhysicalDisk.state is deprecated. '
-                    'Use PhysicalDisk.status instead.')
-        return self.status
-
-    @property
-    def raid_state(self):
-        LOG.warning('PhysicalDisk.raid_state is deprecated. '
-                    'Use PhysicalDisk.raid_status instead.')
-        return self.raid_status
-
 RAIDController = collections.namedtuple(
     'RAIDController', ['id', 'description', 'manufacturer', 'model',
-                       'primary_status', 'firmware_version', 'bus'])
+                       'primary_status', 'firmware_version', 'bus',
+                       'supports_realtime'])
 
-VirtualDiskTuple = collections.namedtuple(
+VirtualDisk = collections.namedtuple(
     'VirtualDisk',
     ['id', 'name', 'description', 'controller', 'raid_level', 'size_mb',
      'status', 'raid_status', 'span_depth', 'span_length',
      'pending_operations', 'physical_disks'])
 
-
-class VirtualDisk(VirtualDiskTuple):
-
-    def __new__(cls, **kwargs):
-        if 'state' in kwargs:
-            LOG.warning('VirtualDisk.state is deprecated. '
-                        'Use VirtualDisk.status instead.')
-            kwargs['status'] = kwargs['state']
-            del kwargs['state']
-
-        if 'raid_state' in kwargs:
-            LOG.warning('VirtualDisk.raid_state is deprecated. '
-                        'Use VirtualDisk.raid_status instead.')
-            kwargs['raid_status'] = kwargs['raid_state']
-            del kwargs['raid_state']
-
-        return super(VirtualDisk, cls).__new__(cls, **kwargs)
-
-    @property
-    def state(self):
-        LOG.warning('VirtualDisk.state is deprecated. '
-                    'Use VirtualDisk.status instead.')
-        return self.status
-
-    @property
-    def raid_state(self):
-        LOG.warning('VirtualDisk.raid_state is deprecated. '
-                    'Use VirtualDisk.raid_status instead.')
-        return self.raid_status
+NO_FOREIGN_DRIVE = "STOR018"
 
 
 class RAIDManagement(object):
 
-    NOT_SUPPORTED_MSG = " operation is not supported on th"
-
     def __init__(self, client):
         """Creates RAIDManagement object
 
@@ -193,7 +140,10 @@ class RAIDManagement(object):
                                                'PrimaryStatus')],
             firmware_version=self._get_raid_controller_attr(
                 drac_controller, 'ControllerFirmwareVersion'),
-            bus=self._get_raid_controller_attr(drac_controller, 'Bus'))
+            bus=self._get_raid_controller_attr(drac_controller, 'Bus'),
+            supports_realtime=RAID_CONTROLLER_IS_REALTIME[
+                self._get_raid_controller_attr(
+                    drac_controller, 'RealtimeCapability')])
 
     def _get_raid_controller_attr(self, drac_controller, attr_name):
         return utils.get_wsman_resource_attr(
@@ -348,16 +298,11 @@ class RAIDManagement(object):
         Disks can be enabled or disabled for RAID mode.
 
         :param physical_disks: list of FQDD ID strings of the physical disks
-               to update
+                               to update
         :param raid_enable: boolean flag, set to True if the disk is to
-               become part of the RAID.  The same flag is applied to all
-               listed disks
+                            become part of the RAID.  The same flag is applied
+                            to all listed disks
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete disk conversion.
@@ -379,7 +324,6 @@ class RAIDManagement(object):
                                  expected_return_value=utils.RET_SUCCESS)
 
         return utils.build_return_dict(doc, uris.DCIM_RAIDService,
-                                       include_commit_required=True,
                                        is_commit_required_value=True)
 
     def create_virtual_disk(self, raid_controller, physical_disks, raid_level,
@@ -398,11 +342,6 @@ class RAIDManagement(object):
         :param span_length: number of disks per span (optional)
         :param span_depth: number of spans in virtual disk (optional)
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete virtual disk creation.
@@ -477,7 +416,6 @@ class RAIDManagement(object):
                                  expected_return_value=utils.RET_SUCCESS)
 
         return utils.build_return_dict(doc, uris.DCIM_RAIDService,
-                                       include_commit_required=True,
                                        is_commit_required_value=True)
 
     def delete_virtual_disk(self, virtual_disk):
@@ -488,11 +426,6 @@ class RAIDManagement(object):
 
         :param virtual_disk: id of the virtual disk
         :returns: a dictionary containing:
-                 - The commit_required key with a boolean value indicating
-                   whether a config job must be created for the values to be
-                   applied.  This key actually has a value that indicates if
-                   a reboot is required.  This key has been deprecated and
-                   will be removed in a future release.
                  - The is_commit_required key with the value always set to
                    True indicating that a config job must be created to
                    complete virtual disk deletion.
@@ -517,7 +450,6 @@ class RAIDManagement(object):
                                  expected_return_value=utils.RET_SUCCESS)
 
         return utils.build_return_dict(doc, uris.DCIM_RAIDService,
-                                       include_commit_required=True,
                                        is_commit_required_value=True)
 
     def is_jbod_capable(self, raid_controller_fqdd):
@@ -556,24 +488,363 @@ class RAIDManagement(object):
 
             # Try moving a disk in the Ready state to JBOD mode
             try:
-                self.convert_physical_disks(
-                    [ready_disk.id],
-                    False)
+                self.convert_physical_disks([ready_disk.id], False)
                 is_jbod_capable = True
 
                 # Flip the disk back to the Ready state.  This results in the
                 # pending value being reset to nothing, so it effectively
                 # undoes the last command and makes the check non-destructive
-                self.convert_physical_disks(
-                    [ready_disk.id],
-                    True)
+                self.convert_physical_disks([ready_disk.id], True)
             except exceptions.DRACOperationFailed as ex:
                 # Fix for python 3, Exception.message no longer
                 # a valid attribute, str(ex) works for both 2.7
                 # and 3.x
-                if self.NOT_SUPPORTED_MSG in str(ex):
+                if constants.NOT_SUPPORTED_MSG in str(ex):
                     pass
                 else:
                     raise
 
         return is_jbod_capable
+
+    def is_raid_controller(self, raid_controller_fqdd, raid_controllers=None):
+        """Find out if object's fqdd is for a raid controller or not
+
+        :param raid_controller_fqdd: The object's fqdd we are testing to see
+                                     if it is a raid controller or not.
+        :param raid_controllers: A list of RAIDControllers used to check for
+                                 the presence of BOSS cards.  If None, the
+                                 iDRAC will be queried for the list of
+                                 controllers.
+        :returns: boolean, True if the device is a RAID controller,
+                  False if not.
+        """
+        return raid_controller_fqdd.startswith('RAID.') or \
+            self.is_boss_controller(raid_controller_fqdd, raid_controllers)
+
+    def is_boss_controller(self, raid_controller_fqdd, raid_controllers=None):
+        """Find out if a RAID controller a BOSS card or not
+
+        :param raid_controller_fqdd: The object's fqdd we are testing to see
+                                     if it is a BOSS card or not.
+        :param raid_controllers: A list of RAIDController to scan for presence
+                                 of BOSS card, if None the drac will be queried
+                                 for the list of controllers which will then be
+                                 scanned.
+        :returns: boolean, True if the device is a BOSS card, False if not.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        """
+        if raid_controllers is None:
+            raid_controllers = self.list_raid_controllers()
+        boss_raid_controllers = [
+            c.id for c in raid_controllers if c.model.startswith('BOSS')]
+        return raid_controller_fqdd in boss_raid_controllers
+
+    def _check_disks_status(self, mode, physical_disks,
+                            controllers_to_physical_disk_ids):
+        """Find disks that failed, need to be configured, or need no change.
+
+        Inspect all the controllers drives and:
+            - See if there are any disks in a failed or unknown state and raise
+            a ValueException where appropriate.
+            - If a controller has disks that still need to be configured add
+            them to the controllers_to_physical_disk_ids dict for the
+            appropriate controller.
+            - If a disk is already in the appropriate state, do nothing, this
+            function should behave in an idempotent manner.
+
+        :param mode: constants.RaidStatus enumeration used to
+                     determine what raid status to check for.
+        :param physical_disks: all physical disks
+        :param controllers_to_physical_disk_ids: Dictionary of controllers and
+               corresponding disk ids to convert to the requested mode.
+        :returns: a dictionary mapping controller FQDDs to the list of
+                  physical disks that need to be converted for that controller.
+        :raises: ValueError: Exception message will list failed drives and
+                     drives whose state cannot be changed at this time, drive
+                     state is not "ready" or "non-RAID".
+        """
+        controllers_to_physical_disk_ids = copy.deepcopy(
+            controllers_to_physical_disk_ids)
+
+        p_disk_id_to_status = {}
+        for physical_disk in physical_disks:
+            p_disk_id_to_status[physical_disk.id] = physical_disk.raid_status
+        failed_disks = []
+        bad_disks = []
+
+        jbod = constants.RaidStatus.jbod
+        raid = constants.RaidStatus.raid
+        for controller, physical_disk_ids \
+                in controllers_to_physical_disk_ids.items():
+            final_physical_disk_ids = []
+            for physical_disk_id in physical_disk_ids:
+                raid_status = p_disk_id_to_status[physical_disk_id]
+                LOG.debug("RAID status for disk id: %s is: %s",
+                          physical_disk_id, raid_status)
+                if ((mode == jbod and raid_status == "non-RAID") or
+                        (mode == raid and raid_status == "ready")):
+                    # This means the disk is already in the desired state,
+                    # so skip it
+                    continue
+                elif ((mode == jbod and raid_status == "ready") or
+                        (mode == raid and raid_status == "non-RAID")):
+                    # This disk is moving from a state we expect to RAID or
+                    # JBOD, so keep it
+                    final_physical_disk_ids.append(physical_disk_id)
+                elif raid_status == "failed":
+                    failed_disks.append(physical_disk_id)
+                else:
+                    # This disk is in one of many states that we don't know
+                    # what to do with, so pitch it
+                    bad_disks.append("{} ({})".format(physical_disk_id,
+                                                      raid_status))
+
+            controllers_to_physical_disk_ids[controller] = (
+                final_physical_disk_ids)
+
+        if failed_disks or bad_disks:
+            error_msg = ""
+
+            if failed_disks:
+                error_msg += ("The following drives have failed: "
+                              "{failed_disks}.  Manually check the status"
+                              " of all drives and replace as necessary, then"
+                              " try again.").format(
+                                  failed_disks=" ".join(failed_disks))
+
+            if bad_disks:
+                if failed_disks:
+                    error_msg += "\n"
+                error_msg += ("Unable to change the state of the following "
+                              "drives because their status is not ready "
+                              "or non-RAID: {}. Bring up the RAID "
+                              "controller GUI on this node and change the "
+                              "drives' status to ready or non-RAID.").format(
+                                  ", ".join(bad_disks))
+
+            raise ValueError(error_msg)
+
+        return controllers_to_physical_disk_ids
+
+    def change_physical_disk_state(self, mode,
+                                   controllers_to_physical_disk_ids=None):
+        """Convert disks RAID status
+
+        This method intelligently converts the requested physical disks from
+        RAID to JBOD or vice versa.  It does this by only converting the
+        disks that are not already in the correct state.
+
+        :param mode: constants.RaidStatus enumeration that indicates the mode
+                     to change the disks to.
+        :param controllers_to_physical_disk_ids: Dictionary of controllers and
+               corresponding disk ids to convert to the requested mode.
+        :returns: a dictionary containing:
+                  - conversion_results, a dictionary that maps controller ids
+                    to the conversion results for that controller.  The
+                    conversion results are a dict that contains:
+                    - The is_commit_required key with the value always set to
+                      True indicating that a config job must be created to
+                      complete disk conversion.
+                    - The is_reboot_required key with a RebootRequired
+                      enumerated value indicating whether the server must be
+                      rebooted to complete disk conversion.
+                  Also contained in the main dict are the following key/values,
+                  which are deprecated, should not be used, and will be removed
+                  in a future release:
+                  - is_reboot_required, a boolean stating whether a reboot is
+                    required or not.
+                  - commit_required_ids, a list of controller ids that will
+                    need to commit their pending RAID changes via a config job.
+        :raises: DRACOperationFailed on error reported back by the DRAC and the
+                 exception message does not contain NOT_SUPPORTED_MSG constant.
+        :raises: Exception on unknown error.
+        """
+        physical_disks = self.list_physical_disks()
+
+        raid = constants.RaidStatus.raid
+
+        if not controllers_to_physical_disk_ids:
+            controllers_to_physical_disk_ids = collections.defaultdict(list)
+
+            all_controllers = self.list_raid_controllers()
+            for physical_d in physical_disks:
+                # Weed out disks that are not attached to a RAID controller
+                if self.is_raid_controller(physical_d.controller,
+                                           all_controllers):
+                    physical_disk_ids = controllers_to_physical_disk_ids[
+                        physical_d.controller]
+
+                    physical_disk_ids.append(physical_d.id)
+
+        '''Modify controllers_to_physical_disk_ids dict by inspecting desired
+        status vs current status of each controller's disks.
+        Raise exception if there are any failed drives or
+        drives not in status 'ready' or 'non-RAID'
+        '''
+        final_ctls_to_phys_disk_ids = self._check_disks_status(
+                mode, physical_disks, controllers_to_physical_disk_ids)
+
+        is_reboot_required = False
+        controllers = []
+        controllers_to_results = {}
+        for controller, physical_disk_ids \
+                in final_ctls_to_phys_disk_ids.items():
+            if physical_disk_ids:
+                LOG.debug("Converting the following disks to {} on RAID "
+                          "controller {}: {}".format(
+                              mode, controller, str(physical_disk_ids)))
+                try:
+                    conversion_results = \
+                        self.convert_physical_disks(physical_disk_ids,
+                                                    mode == raid)
+                except exceptions.DRACOperationFailed as ex:
+                    if constants.NOT_SUPPORTED_MSG in str(ex):
+                        LOG.debug("Controller {} does not support "
+                                  "JBOD mode".format(controller))
+                        controllers_to_results[controller] = \
+                            utils.build_return_dict(
+                                doc=None,
+                                resource_uri=None,
+                                is_commit_required_value=False,
+                                is_reboot_required_value=constants.
+                                RebootRequired.false)
+                    else:
+                        raise
+                else:
+                    controllers_to_results[controller] = conversion_results
+
+                    # Remove the code below when is_reboot_required and
+                    # commit_required_ids are deprecated
+                    reboot_true = constants.RebootRequired.true
+                    reboot_optional = constants.RebootRequired.optional
+                    _is_reboot_required = \
+                        conversion_results["is_reboot_required"]
+                    is_reboot_required = is_reboot_required \
+                        or (_is_reboot_required
+                            in [reboot_true, reboot_optional])
+                    controllers.append(controller)
+            else:
+                controllers_to_results[controller] = \
+                    utils.build_return_dict(
+                        doc=None,
+                        resource_uri=None,
+                        is_commit_required_value=False,
+                        is_reboot_required_value=constants.
+                        RebootRequired.false)
+
+        return {'conversion_results': controllers_to_results,
+                'is_reboot_required': is_reboot_required,
+                'commit_required_ids': controllers}
+
+    def is_realtime_supported(self, raid_controller_fqdd):
+        """Find if controller supports realtime or not
+
+        :param raid_controller_fqdd: ID of RAID controller
+        :returns: True or False
+        """
+        drac_raid_controllers = self.list_raid_controllers()
+        realtime_controller = [cnt.id for cnt in drac_raid_controllers
+                               if cnt.supports_realtime]
+
+        if raid_controller_fqdd in realtime_controller:
+            return True
+
+        return False
+
+    def reset_raid_config(self, raid_controller):
+        """Delete all virtual disk and unassign all hotspares
+
+        The job to reset the RAID controller config will be in pending state.
+        For the changes to be applied, a config job must be created.
+
+        :param raid_controller: id of the RAID controller
+        :returns: a dictionary containing:
+                 - The is_commit_required key with the value always set to
+                   True indicating that a config job must be created to
+                   reset configuration.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted to
+                   reset configuration.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        """
+
+        selectors = {'SystemCreationClassName': 'DCIM_ComputerSystem',
+                     'CreationClassName': 'DCIM_RAIDService',
+                     'SystemName': 'DCIM:ComputerSystem',
+                     'Name': 'DCIM:RAIDService'}
+        properties = {'Target': raid_controller}
+
+        doc = self.client.invoke(uris.DCIM_RAIDService, 'ResetConfig',
+                                 selectors, properties,
+                                 expected_return_value=utils.RET_SUCCESS)
+
+        return utils.build_return_dict(doc, uris.DCIM_RAIDService,
+                                       is_commit_required_value=True)
+
+    def clear_foreign_config(self, raid_controller):
+        """Free up foreign drives
+
+        The job to clear foreign config will be in pending state.
+        For the changes to be applied, a config job must be created.
+
+        :param raid_controller: id of the RAID controller
+        :returns: a dictionary containing:
+                 - The is_commit_required key with the value always set to
+                   True indicating that a config job must be created to
+                   clear foreign configuration.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted to
+                   clear foreign configuration.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                 interface
+        :raises: DRACUnexpectedReturnValue on return value mismatch
+        """
+
+        selectors = {'SystemCreationClassName': 'DCIM_ComputerSystem',
+                     'CreationClassName': 'DCIM_RAIDService',
+                     'SystemName': 'DCIM:ComputerSystem',
+                     'Name': 'DCIM:RAIDService'}
+        properties = {'Target': raid_controller}
+
+        doc = self.client.invoke(uris.DCIM_RAIDService, 'ClearForeignConfig',
+                                 selectors, properties,
+                                 check_return_value=False)
+
+        is_commit_required_value = True
+        is_reboot_required_value = None
+
+        ret_value = utils.find_xml(doc,
+                                   'ReturnValue',
+                                   uris.DCIM_RAIDService).text
+
+        if ret_value == utils.RET_ERROR:
+            message_id = utils.find_xml(doc,
+                                        'MessageID',
+                                        uris.DCIM_RAIDService).text
+
+            # A MessageID 'STOR018' indicates no foreign drive was
+            # detected. Return a value which informs the caller nothing
+            # further needs to be done.
+            if message_id == NO_FOREIGN_DRIVE:
+                is_commit_required_value = False
+                is_reboot_required_value = constants.RebootRequired.false
+            else:
+                message = utils.find_xml(doc,
+                                         'Message',
+                                         uris.DCIM_RAIDService).text
+                raise exceptions.DRACOperationFailed(
+                        drac_messages=message)
+
+        return utils.build_return_dict(
+                doc, uris.DCIM_RAIDService,
+                is_commit_required_value=is_commit_required_value,
+                is_reboot_required_value=is_reboot_required_value)
