@@ -18,6 +18,7 @@ import mock
 import requests_mock
 
 import dracclient.client
+from dracclient import constants
 from dracclient import exceptions
 from dracclient.resources import bios
 import dracclient.resources.job
@@ -347,10 +348,13 @@ class ClientBIOSConfigurationTestCase(base.BaseTest):
         result = self.drac_client.set_bios_settings(
             {'ProcVirtualization': 'Disabled'})
 
-        self.assertEqual({'commit_required': True}, result)
+        self.assertEqual({'is_commit_required': True,
+                          'is_reboot_required': constants.RebootRequired.true},
+                         result)
         mock_invoke.assert_called_once_with(
             mock.ANY, uris.DCIM_BIOSService, 'SetAttributes',
-            expected_selectors, expected_properties)
+            expected_selectors, expected_properties,
+            wait_for_idrac=True)
 
     def test_set_bios_settings_error(self, mock_requests,
                                      mock_wait_until_idrac_is_ready):
@@ -394,7 +398,10 @@ class ClientBIOSConfigurationTestCase(base.BaseTest):
         result = self.drac_client.set_bios_settings(
             {'ProcVirtualization': 'Enabled'})
 
-        self.assertEqual({'commit_required': False}, result)
+        self.assertEqual({'is_commit_required': False,
+                          'is_reboot_required':
+                          constants.RebootRequired.false},
+                         result)
 
     def test_set_bios_settings_with_readonly_attr(
             self, mock_requests, mock_wait_until_idrac_is_ready):
@@ -476,7 +483,8 @@ class ClientBIOSChangesTestCase(base.BaseTest):
         mock_create_config_job.assert_called_once_with(
             mock.ANY, resource_uri=uris.DCIM_BIOSService,
             cim_creation_class_name='DCIM_BIOSService',
-            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1', reboot=False)
+            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1',
+            reboot=False, start_time='TIME_NOW')
 
     @mock.patch.object(dracclient.resources.job.JobManagement,
                        'create_config_job', spec_set=True, autospec=True)
@@ -487,7 +495,38 @@ class ClientBIOSChangesTestCase(base.BaseTest):
         mock_create_config_job.assert_called_once_with(
             mock.ANY, resource_uri=uris.DCIM_BIOSService,
             cim_creation_class_name='DCIM_BIOSService',
-            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1', reboot=True)
+            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1',
+            reboot=True, start_time='TIME_NOW')
+
+    @mock.patch.object(dracclient.resources.job.JobManagement,
+                       'create_config_job', spec_set=True, autospec=True)
+    def test_commit_pending_bios_changes_with_time(
+            self, mock_create_config_job):
+        timestamp = '20140924140201'
+        self.drac_client.commit_pending_bios_changes(
+            start_time=timestamp)
+
+        mock_create_config_job.assert_called_once_with(
+            mock.ANY, resource_uri=uris.DCIM_BIOSService,
+            cim_creation_class_name='DCIM_BIOSService',
+            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1',
+            reboot=False, start_time=timestamp)
+
+    @mock.patch.object(dracclient.resources.job.JobManagement,
+                       'create_config_job', spec_set=True, autospec=True)
+    def test_commit_pending_bios_changes_with_reboot_and_time(
+            self,
+            mock_create_config_job):
+        timestamp = '20140924140201'
+        self.drac_client.commit_pending_bios_changes(
+            reboot=True,
+            start_time=timestamp)
+
+        mock_create_config_job.assert_called_once_with(
+            mock.ANY, resource_uri=uris.DCIM_BIOSService,
+            cim_creation_class_name='DCIM_BIOSService',
+            cim_name='DCIM:BIOSService', target='BIOS.Setup.1-1',
+            reboot=True, start_time=timestamp)
 
     @mock.patch.object(dracclient.resources.job.JobManagement,
                        'delete_pending_config', spec_set=True, autospec=True)
