@@ -133,6 +133,14 @@ JobInvocations = {
             'error': load_wsman_xml(
                 'bios_service-invoke-delete_pending_configuration-error'),
         },
+    },
+    uris.DCIM_LCService: {
+        'CreateConfigJob': {
+            'ok': load_wsman_xml(
+                'lc_service-invoke-create_config_job-ok'),
+            'error': load_wsman_xml(
+                'lc_service-invoke-create_config_job-error'),
+        },
     }
 }
 
@@ -164,7 +172,14 @@ iDracCardInvocations = {
         'SetAttributes': {
             'ok': load_wsman_xml(
                 'idrac_service-invoke-set_attributes-ok')
+        },
+        'iDRACReset': {
+            'ok': load_wsman_xml(
+                'idrac_service-reset-ok'),
+            'error': load_wsman_xml(
+                'idrac_service-reset-error')
         }
+
     }
 }
 
@@ -185,7 +200,15 @@ LifecycleControllerInvocations = {
         'GetRemoteServicesAPIStatus': {
             'is_ready': load_wsman_xml('lc_getremoteservicesapistatus_ready'),
             'is_not_ready': load_wsman_xml(
-                'lc_getremoteservicesapistatus_not_ready')
+                'lc_getremoteservicesapistatus_not_ready'),
+            'is_recovery': load_wsman_xml(
+                'lc_getremoteservicesapistatus_recovery'),
+        },
+        'SetAttributes': {
+            'ok': load_wsman_xml(
+                'lc_service-invoke-set_attributes-ok'),
+            'error': load_wsman_xml(
+                'lc_service-invoke-set_attributes-error'),
         }
     }
 }
@@ -246,6 +269,20 @@ RAIDInvocations = {
                 'raid_service-invoke-convert_physical_disks-ok'),
             'error': load_wsman_xml(
                 'raid_service-invoke-convert_physical_disks-error'),
+        },
+        'ResetConfig': {
+            'ok': load_wsman_xml(
+                'raid_service-invoke-reset_raid_config-ok'),
+            'error': load_wsman_xml(
+                'raid_service-invoke-reset_raid_config-error'),
+        },
+        'ClearForeignConfig': {
+            'ok': load_wsman_xml(
+                'raid_service-invoke-clear_foreign_config-ok'),
+            'no_foreign_drive': load_wsman_xml(
+                'raid_service-invoke-clear_foreign_config-no_foreign_drive'),
+            'invalid_controller_id': load_wsman_xml(
+                'raid_service-invoke-clear_foreign_config-invalid_controller'),
         }
     }
 }
