@@ -132,9 +132,7 @@ def get_all_wsman_resource_attrs(doc, resource_uri, attr_name, nullable=False):
 
 def build_return_dict(doc, resource_uri,
                       is_commit_required_value=None,
-                      is_reboot_required_value=None,
-                      commit_required_value=None,
-                      include_commit_required=False):
+                      is_reboot_required_value=None):
     """Builds a dictionary to be returned
 
        Build a dictionary to be returned from WSMAN operations that are not
@@ -148,17 +146,9 @@ def build_return_dict(doc, resource_uri,
     :param is_reboot_required_value: The value to be returned for
            is_reboot_required, or None if the value should be determined
            from the doc.
-    :param commit_required_value: The value to be returned for
-           commit_required, or None if the value should be determined
-           from the doc.
-    :parm include_commit_required: Indicates if the deprecated commit_required
-                                   should be returned in the result.
     :returns: a dictionary containing:
              - is_commit_required: indicates if a commit is required.
              - is_reboot_required: indicates if a reboot is required.
-             - commit_required: a deprecated key indicating if a commit is
-               required.  This key actually has a value that indicates if a
-               reboot is required.
     """
 
     if is_reboot_required_value is not None and \
@@ -180,14 +170,6 @@ def build_return_dict(doc, resource_uri,
 
     result['is_reboot_required'] = is_reboot_required_value
 
-    # Include commit_required in the response for backwards compatibility
-    # TBD: Remove this parameter in the future
-    if include_commit_required:
-        if commit_required_value is None:
-            commit_required_value = is_reboot_required(doc, resource_uri)
-
-        result['commit_required'] = commit_required_value
-
     return result
 
 
@@ -251,7 +233,7 @@ def validate_integer_value(value, attr_name, error_msgs):
 
 
 def list_settings(client, namespaces, by_name=True, fqdd_filter=None,
-                  name_formatter=None):
+                  name_formatter=None, wait_for_idrac=True):
     """List the configuration settings
 
     :param client: an instance of WSManClient.
@@ -263,6 +245,9 @@ def list_settings(client, namespaces, by_name=True, fqdd_filter=None,
     :param name_formatter: a method used to format the keys in the
                            returned dictionary.  By default,
                            attribute.name will be used.
+    :param wait_for_idrac: indicates whether or not to wait for the
+                           iDRAC to be ready to accept commands before
+                           issuing the command.
     :returns: a dictionary with the settings using name or instance_id as
               the key.
     :raises: WSManRequestFailure on request failures
@@ -274,7 +259,7 @@ def list_settings(client, namespaces, by_name=True, fqdd_filter=None,
     result = {}
     for (namespace, attr_cls) in namespaces:
         attribs = _get_config(client, namespace, attr_cls, by_name,
-                              fqdd_filter, name_formatter)
+                              fqdd_filter, name_formatter, wait_for_idrac)
         if not set(result).isdisjoint(set(attribs)):
             raise exceptions.DRACOperationFailed(
                 drac_messages=('Colliding attributes %r' % (
@@ -284,10 +269,10 @@ def list_settings(client, namespaces, by_name=True, fqdd_filter=None,
 
 
 def _get_config(client, resource, attr_cls, by_name, fqdd_filter,
-                name_formatter):
+                name_formatter, wait_for_idrac):
     result = {}
 
-    doc = client.enumerate(resource)
+    doc = client.enumerate(resource, wait_for_idrac=wait_for_idrac)
     items = doc.find('.//{%s}Items' % wsman.NS_WSMAN)
 
     for item in items:
@@ -316,7 +301,7 @@ def set_settings(settings_type,
                  cim_name,
                  target,
                  name_formatter=None,
-                 include_commit_required=False):
+                 wait_for_idrac=True):
     """Generically handles setting various types of settings on the iDRAC
 
     This method pulls the current list of settings from the iDRAC then compares
@@ -337,14 +322,10 @@ def set_settings(settings_type,
     :param name_formatter: a method used to format the keys in the
                            returned dictionary.  By default,
                            attribute.name will be used.
-    :parm include_commit_required: Indicates if the deprecated commit_required
-                                   should be returned in the result.
+    :param wait_for_idrac: indicates whether or not to wait for the
+                           iDRAC to be ready to accept commands before issuing
+                           the command
     :returns: a dictionary containing:
-             - The commit_required key with a boolean value indicating
-               whether a config job must be created for the values to be
-               applied.  This key actually has a value that indicates if
-               a reboot is required.  This key has been deprecated and
-               will be removed in a future release.
              - The is_commit_required key with a boolean value indicating
                whether a config job must be created for the values to be
                applied.
@@ -361,12 +342,15 @@ def set_settings(settings_type,
     """
 
     current_settings = list_settings(client, namespaces, by_name=True,
-                                     name_formatter=name_formatter)
+                                     name_formatter=name_formatter,
+                                     wait_for_idrac=wait_for_idrac)
 
     unknown_keys = set(new_settings) - set(current_settings)
     if unknown_keys:
-        msg = ('Unknown %(settings_type)s attributes found: %(unknown_keys)r' %
-               {'settings_type': settings_type, 'unknown_keys': unknown_keys})
+        msg = ('Unknown %(settings_type)s attributes found: '
+               '%(unknown_keys)r' %
+               {'settings_type': settings_type,
+                'unknown_keys': unknown_keys})
         raise exceptions.InvalidParameterValue(reason=msg)
 
     read_only_keys = []
@@ -412,21 +396,21 @@ def set_settings(settings_type,
         return build_return_dict(
             None,
             resource_uri,
-            include_commit_required=include_commit_required,
             is_commit_required_value=False,
-            is_reboot_required_value=constants.RebootRequired.false,
-            commit_required_value=False)
+            is_reboot_required_value=constants.RebootRequired.false)
 
     selectors = {'CreationClassName': cim_creation_class_name,
                  'Name': cim_name,
                  'SystemCreationClassName': 'DCIM_ComputerSystem',
                  'SystemName': 'DCIM:ComputerSystem'}
+
     properties = {'Target': target,
                   'AttributeName': attrib_names,
                   'AttributeValue': [new_settings[attr] for attr
                                      in attrib_names]}
-    doc = client.invoke(resource_uri, 'SetAttributes',
-                        selectors, properties)
 
-    return build_return_dict(doc, resource_uri,
-                             include_commit_required=include_commit_required)
+    doc = client.invoke(resource_uri, 'SetAttributes',
+                        selectors, properties,
+                        wait_for_idrac=wait_for_idrac)
+
+    return build_return_dict(doc, resource_uri)
